/*
 * Copyright (c) 2011, Endea.org or third-party contributors as
 * indicated by the @author tags.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

package endea.engine.internal;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import org.apache.maven.repository.internal.DefaultServiceLocator;
import org.apache.maven.repository.internal.MavenRepositorySystemSession;
import org.sonatype.aether.RepositorySystem;
import org.sonatype.aether.RepositorySystemSession;
import org.sonatype.aether.collection.CollectRequest;
import org.sonatype.aether.collection.DependencyCollectionException;
import org.sonatype.aether.connector.async.AsyncRepositoryConnectorFactory;
import org.sonatype.aether.graph.Dependency;
import org.sonatype.aether.graph.DependencyNode;
import org.sonatype.aether.repository.LocalRepository;
import org.sonatype.aether.repository.RemoteRepository;
import org.sonatype.aether.resolution.DependencyRequest;
import org.sonatype.aether.resolution.DependencyResolutionException;
import org.sonatype.aether.spi.connector.RepositoryConnectorFactory;
import org.sonatype.aether.util.artifact.JavaScopes;
import org.sonatype.aether.util.graph.PreorderNodeListGenerator;

import endea.engine.Configuration;

/**
 * @author David de Mingo
 */
public class Aether {

	public static List<URL> resolve(Configuration config)
			throws DependencyCollectionException,
			DependencyResolutionException, MalformedURLException {

		List<URL> urls = new ArrayList<URL>();
		for (File file : nodeList(config).getFiles())
			urls.add(file.toURI().toURL());

		return urls;
	}

	public static PreorderNodeListGenerator nodeList(Configuration config)
			throws DependencyCollectionException,
			DependencyResolutionException, MalformedURLException {

		PreorderNodeListGenerator nlg = new PreorderNodeListGenerator();

		RepositorySystem repoSystem = newRepositorySystem();
		RepositorySystemSession session = newSession(repoSystem,
				config.getLocalRepository());

		CollectRequest collectRequest = new CollectRequest();
		collectRequest.setRoot(new Dependency(config.getArtifact(),
				JavaScopes.COMPILE));

		List<RemoteRepository> repositories = new ArrayList<RemoteRepository>();
		repositories.add(config.getRemoteRepository());
		collectRequest.setRepositories(repositories);

		DependencyNode node = repoSystem.collectDependencies(session,
				collectRequest).getRoot();
		repoSystem.resolveDependencies(session, new DependencyRequest(node,
				null));
		node.accept(nlg);

		return nlg;
	}

	private static RepositorySystemSession newSession(RepositorySystem system,
			LocalRepository localRepository) {

		MavenRepositorySystemSession session = new MavenRepositorySystemSession();

		session.setLocalRepositoryManager(system
				.newLocalRepositoryManager(localRepository));
		session.setTransferListener(new ConsoleTransferListener());
		// session.setRepositoryListener(new ConsoleRepositoryListener());

		return session;
	}

	private static RepositorySystem newRepositorySystem() {

		DefaultServiceLocator locator = new DefaultServiceLocator();
		locator.addService(RepositoryConnectorFactory.class,
				AsyncRepositoryConnectorFactory.class);

		return locator.getService(RepositorySystem.class);
	}
}
